#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright Sensors & Signals LLC https://www.snstac.com/
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""DJICOT Class Tests."""

import asyncio
import unittest
from unittest.mock import patch
from djicot.classes import DJIWorker
from djicot.constants import DEFAULT_SENSOR_COT_TYPE, DEFAULT_COT_TYPE
from configparser import ConfigParser, SectionProxy
import pytest


@pytest.fixture
def config():
    config_parser = ConfigParser()
    config_parser.read_dict(
        {
            "DEFAULT": {
                "INCLUDE_TISB": "false",
                "TISB_ONLY": "false",
                "INCLUDE_ALL_CRAFT": "true",
            }
        }
    )
    return config_parser["DEFAULT"]


@pytest.fixture
def real_queue():
    return asyncio.Queue()


@pytest.fixture
def real_queue2():
    return asyncio.Queue()


@pytest.fixture
def real_worker(real_queue, real_queue2, config):
    return DJIWorker(real_queue, config, real_queue)


class MockWriter:
    """Mock CoT Event Writer."""

    def __init__(self):
        self.events = []

    async def send(self, event):
        self.events.append(event)


@pytest.fixture
def mock_writer():
    return MockWriter()


@pytest.mark.asyncio
async def test_handle_data_with_invalid_data(real_worker, real_queue):
    """Test handle_data method."""
    data = b"<dji_data/>"
    await real_worker.handle_data(data)

    event = await real_queue.get()
    print(event)
    assert b"DJICOT" in event
    assert DEFAULT_COT_TYPE.encode("UTF-8") in event


@pytest.mark.asyncio
async def test_hello_event(real_worker, real_queue):
    """Test hello_event method."""
    result = await real_worker.hello_event(init=True)
    event = await real_queue.get()
    print(event)
    assert event is not None
    assert b"CUAS" in event
    assert result is None


@pytest.mark.asyncio
async def _test_handle_data_with_valid_data(real_worker, real_queue):
    """Test handle_data method with valid data."""
    data = b"<dji_data/>"
    with patch(
        "djicot.functions.gen_dji_cot", return_value=b"<cot_event/>"
    ) as mock_func:
        result = await real_worker.handle_data(data)
        event = await real_queue.get()
        assert event == b"<cot_event/>"
        assert result is None
        mock_func.assert_any_call(data, real_worker.config, "gen_dji_cot")
        mock_func.assert_any_call(data, real_worker.config, "dji_uas_to_cot")


# @pytest.mark.asyncio
# async def test_handle_data_with_invalid_data(real_worker, real_queue):
#     """Test handle_data method with invalid data."""
#     data = b"<invalid_data/>"
#     with patch(
#         "djicot.classes.gen_dji_cot", side_effect=[None, b"<cot_event/>"]
#     ) as mock_func:
#         result = await real_worker.handle_data(data)
#         event = await real_queue.get()
#         assert event == b"<cot_event/>"
#         assert result is None
#         assert mock_func.call_count == 2
#         mock_func.assert_any_call(data, real_worker.config, "gen_dji_cot")
#         mock_func.assert_any_call(data, real_worker.config, "dji_uas_to_cot")


# @pytest.mark.asyncio
# async def test_handle_data_with_no_event(real_worker, real_queue):
#     """Test handle_data method with no event generated."""
#     data = b"<no_event_data/>"
#     with patch("djicot.classes.xml_to_cot", return_value=None) as mock_xml_to_cot:
#         result = await real_worker.handle_data(data)
#         # assert real_queue.empty()
#         assert result is None
#         assert mock_xml_to_cot.call_count == 2
#         mock_xml_to_cot.assert_any_call(data, real_worker.config, "dji_to_cot")
#         mock_xml_to_cot.assert_any_call(data, real_worker.config, "dji_range_to_cot")


# @pytest.mark.asyncio
# async def test_handle_data_invalid_decode(real_worker, real_queue):
#     """Test handle_data method with invalid decode."""
#     data = {
#         "serial_number": "3NZCJ290048m",
#         "device_type": "",
#         "device_type_8": 0,
#         "op_lat": 1671.8222513793953,
#         "op_lon": 7421.5640767075565,
#         "uas_lat": -9928.970842190301,
#         "uas_lon": -7990.694745406313,
#         "height": -2221.4,
#         "altitude": 2561.7,
#         "home_lat": 4109.836907633514,
#         "home_lon": 6294.859000876625,
#         "freq": 5796.5,
#         "speed_e": -3020.9,
#         "speed_n": 301.8,
#         "speed_u": 1307.4,
#         "rssi": -58,
#     }
#     result = await real_worker.handle_data(data)
#     assert result is None
#     # assert real_queue.empty()


if __name__ == "__main__":
    unittest.main()



import unittest
from unittest.mock import patch
from djicot.commands import main

#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright Sensors & Signals LLC https://www.snstac.com/
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""Tests for djicot.commands."""


class TestCommands(unittest.TestCase):
    @patch("pytak.cli")
    def test_main(self, mock_cli):
        """Test the main function."""
        main()
        mock_cli.assert_called_once_with("djicot")


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright Sensors & Signals LLC https://www.snstac.com/
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""DJICOT Function Tests."""


import struct
import unittest
from djicot.dji_functions import parse_frame, parse_data


class DJIFunctionsTestCase(unittest.TestCase):
    """
    Test class for functions... functions.
    """

    def test_parse_frame(self):
        """Test parse_frame function with valid frame data."""
        frame = bytearray(10)
        frame[:2] = b"\x01\x02"  # frame_header
        frame[2] = 0x03  # package_type
        frame[3:5] = struct.pack("H", 10)  # package_length
        frame[5:10] = b"hello"  # data

        package_type, data = parse_frame(frame)
        self.assertEqual(package_type, 0x03)
        self.assertEqual(data, b"hello")

    def test_parse_frame_with_invalid_length(self):
        """Test parse_frame function with invalid length."""
        frame = bytearray(8)
        frame[:2] = b"\x01\x02"  # frame_header
        frame[2] = 0x03  # package_type
        frame[3:5] = struct.pack("H", 10)  # package_length
        frame[5:8] = b"hel"  # data

        package_type, data = parse_frame(frame)
        self.assertEqual(package_type, 0x03)
        self.assertEqual(data, b"hel")

    def test_parse_data(self):
        """Test parse_data function with valid data."""

        data = bytearray(227)
        data[:64] = b"serial_number".ljust(64, b"\x00")
        data[64:128] = b"device_type".ljust(64, b"\x00")
        data[128] = 8
        data[129:137] = struct.pack("d", 37.7749)  # app_lat
        data[137:145] = struct.pack("d", -122.4194)  # app_lon
        data[145:153] = struct.pack("d", 37.7749)  # uas_lat
        data[153:161] = struct.pack("d", -122.4194)  # uas_lon
        data[161:169] = struct.pack("d", 100.0)  # height
        data[169:177] = struct.pack("d", 200.0)  # altitude
        data[177:185] = struct.pack("d", 37.7749)  # home_lat
        data[185:193] = struct.pack("d", -122.4194)  # home_lon
        data[193:201] = struct.pack("d", 2.4)  # freq
        data[201:209] = struct.pack("d", 10.0)  # speed_e
        data[209:217] = struct.pack("d", 5.0)  # speed_n
        data[217:225] = struct.pack("d", 1.0)  # speed_u
        data[225:227] = struct.pack("h", -50)  # rssi

        parsed_data = parse_data(data)
        self.assertEqual(parsed_data.get("serial_number"), "serial_number")
        self.assertEqual(parsed_data.get("device_type"), "device_type")
        self.assertEqual(parsed_data.get("device_type_8"), 8)
        self.assertEqual(parsed_data.get("op_lat"), 37.7749)
        self.assertEqual(parsed_data.get("op_lon"), -122.4194)
        self.assertEqual(parsed_data.get("uas_lat"), 37.7749)
        self.assertEqual(parsed_data.get("uas_lon"), -122.4194)
        self.assertEqual(parsed_data.get("height"), 100.0)
        self.assertEqual(parsed_data.get("altitude"), 200.0)
        self.assertEqual(parsed_data.get("home_lat"), 37.7749)
        self.assertEqual(parsed_data.get("home_lon"), -122.4194)
        self.assertEqual(parsed_data.get("freq"), 2.4)
        self.assertEqual(parsed_data.get("speed_e"), 10.0)
        self.assertEqual(parsed_data.get("speed_n"), 5.0)
        self.assertEqual(parsed_data.get("speed_u"), 1.0)
        self.assertEqual(parsed_data.get("rssi"), -50)

    def _test_parse_data_with_invalid_data(self):
        """Test parse_data function with invalid data."""
        data = b"invalid_data"
        parsed_data = parse_data(data)
        self.assertEqual(
            parsed_data["device_type"],
            "Unknown DJI OcuSync Format (Encrypted or Partial Data)",
        )
        self.assertEqual(parsed_data["device_type_8"], 255)

    def test_parse_frame_with_package_type_0x01(self):
        """Test parse_frame function with package_type 0x01."""
        frame = bytearray(10)
        frame[:2] = b"\x01\x02"  # frame_header
        frame[2] = 0x01  # package_type
        frame[3:5] = struct.pack("H", 10)  # package_length
        frame[5:10] = b"hello"  # data

        package_type, data = parse_frame(frame)
        self.assertEqual(package_type, 0x01)
        self.assertEqual(data, b"hello")


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright Sensors & Signals LLC https://www.snstac.com/
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""DJICOT Function Tests."""

import struct
import unittest

import xml.etree.ElementTree as ET

import djicot
import djicot.dji_functions


class FunctionsTestCase(unittest.TestCase):
    """
    Test class for functions... functions.
    """

    def test_parse_data(self):
        """Test parse_data function with valid data."""

        data = bytearray(227)
        data[:64] = b"serial_number".ljust(64, b"\x00")
        data[64:128] = b"device_type".ljust(64, b"\x00")
        data[128] = 8
        data[129:137] = struct.pack("d", 37.7749)  # app_lat
        data[137:145] = struct.pack("d", -122.4194)  # app_lon
        data[145:153] = struct.pack("d", 37.7749)  # uas_lat
        data[153:161] = struct.pack("d", -122.4194)  # uas_lon
        data[161:169] = struct.pack("d", 100.0)  # height
        data[169:177] = struct.pack("d", 200.0)  # altitude
        data[177:185] = struct.pack("d", 37.7749)  # home_lat
        data[185:193] = struct.pack("d", -122.4194)  # home_lon
        data[193:201] = struct.pack("d", 2.4)  # freq
        data[201:209] = struct.pack("d", 10.0)  # speed_e
        data[209:217] = struct.pack("d", 5.0)  # speed_n
        data[217:225] = struct.pack("d", 1.0)  # speed_u
        data[225:227] = struct.pack("h", -50)  # rssi

        parsed_data = djicot.dji_functions.parse_data(data)
        print("parsed_data")
        print(parsed_data)
        assert parsed_data.get("serial_number") == "serial_number"
        assert parsed_data.get("device_type") == "device_type"
        assert parsed_data.get("device_type_8") == 8
        assert parsed_data.get("op_lat") == 37.7749
        assert parsed_data.get("op_lon") == -122.4194
        assert parsed_data.get("uas_lat") == 37.7749
        assert parsed_data.get("uas_lon") == -122.4194
        assert parsed_data.get("height") == 100.0
        assert parsed_data.get("altitude") == 200.0
        assert parsed_data.get("home_lat") == 37.7749
        assert parsed_data.get("home_lon") == -122.4194
        assert parsed_data.get("freq") == 2.4
        assert parsed_data.get("speed_e") == 10.0
        assert parsed_data.get("speed_n") == 5.0
        assert parsed_data.get("speed_u") == 1.0
        assert parsed_data.get("rssi") == -50

    def _test_parse_data_with_invalid_data(self):
        """Test parse_data function with invalid data."""
        data = b"invalid_data"
        parsed_data = djicot.dji_functions.parse_data(data)
        assert parsed_data["device_type"] == "Got a DJI drone with encryption"
        assert parsed_data["device_type_8"] == 255

    def test_parse_frame(self):
        """Test parse_frame function with valid frame data."""
        frame = bytearray(10)
        frame[:2] = b"\x01\x02"  # frame_header
        frame[2] = 0x03  # package_type
        frame[3:5] = struct.pack("H", 10)  # package_length
        frame[5:10] = b"hello"  # data

        package_type, data = djicot.functions.parse_frame(frame)
        assert package_type == 0x03
        assert data == b"hello"

    def test_parse_frame_with_invalid_length(self):
        """Test parse_frame function with invalid length."""
        frame = bytearray(8)
        frame[:2] = b"\x01\x02"  # frame_header
        frame[2] = 0x03  # package_type
        frame[3:5] = struct.pack("H", 10)  # package_length
        frame[5:8] = b"hel"  # data

        package_type, data = djicot.functions.parse_frame(frame)
        assert package_type == 0x03
        assert data == b"hel"

    def test_handle_frame_valid(self):
        """Test handle_frame function with valid frame data."""
        frame = bytearray(10)
        frame[:2] = b"\x01\x02"  # frame_header
        frame[2] = 0x01  # package_type

        data = bytearray(227)
        data[:64] = b"F4XF7249QM06Q26P".ljust(64, b"\x00")
        data[64:128] = b"DJI Mini 3 pro".ljust(64, b"\x00")
        data[128] = 73
        data[129:137] = struct.pack("d", 37.7749)  # app_lat
        data[137:145] = struct.pack("d", -122.4194)  # app_lon
        data[145:153] = struct.pack("d", 37.7749)  # uas_lat
        data[153:161] = struct.pack("d", -122.4194)  # uas_lon
        data[161:169] = struct.pack("d", 100.0)  # height
        data[169:177] = struct.pack("d", 200.0)  # altitude
        data[177:185] = struct.pack("d", 37.7749)  # home_lat
        data[185:193] = struct.pack("d", -122.4194)  # home_lon
        data[193:201] = struct.pack("d", 2.4)  # freq
        data[201:209] = struct.pack("d", 10.0)  # speed_e
        data[209:217] = struct.pack("d", 5.0)  # speed_n
        data[217:225] = struct.pack("d", 1.0)  # speed_u
        data[225:227] = struct.pack("h", -50)  # rssi

        frame[3:5] = struct.pack("H", len(data))  # package_length
        frame[5 : 5 + len(data)] = data

        config = {
            "DEBUG": True,
            "COT_TYPE": "a-f-G-U-C",
            "COT_STALE": 600,
            "COT_HOST_ID": "test_host",
            "SENSOR_ID": "test_sensor",
            "SENSOR_SN": "test_sn",
            "SENSOR_TYPE": "test_type",
            "SENSOR_NAME": "test_name",
            "COT_ACCESS": "test_access",
        }

        events = djicot.functions.handle_frame(frame, config)
        assert len(events) == 1
        for event in events:
            assert djicot.DEFAULT_COT_TYPE in event.decode("utf-8")

    def test_handle_frame_invalid_package_type(self):
        """Test handle_frame function with invalid package type."""
        frame = bytearray(10)
        frame[:2] = b"\x01\x02"  # frame_header
        frame[2] = 0x02  # invalid package_type

        data = bytearray(227)
        data[:64] = b"F4XF7249QM06Q26P".ljust(64, b"\x00")
        data[64:128] = b"DJI Mini 3 pro".ljust(64, b"\x00")
        data[128] = 73
        data[129:137] = struct.pack("d", 37.7749)  # app_lat
        data[137:145] = struct.pack("d", -122.4194)  # app_lon
        data[145:153] = struct.pack("d", 37.7749)  # uas_lat
        data[153:161] = struct.pack("d", -122.4194)  # uas_lon
        data[161:169] = struct.pack("d", 100.0)  # height
        data[169:177] = struct.pack("d", 200.0)  # altitude
        data[177:185] = struct.pack("d", 37.7749)  # home_lat
        data[185:193] = struct.pack("d", -122.4194)  # home_lon
        data[193:201] = struct.pack("d", 2.4)  # freq
        data[201:209] = struct.pack("d", 10.0)  # speed_e
        data[209:217] = struct.pack("d", 5.0)  # speed_n
        data[217:225] = struct.pack("d", 1.0)  # speed_u
        data[225:227] = struct.pack("h", -50)  # rssi

        frame[3:5] = struct.pack("H", len(data))  # package_length
        frame[5 : 5 + len(data)] = data

        config = {
            "DEBUG": True,
            "COT_TYPE": "a-f-G-U-C",
            "COT_STALE": 600,
            "COT_HOST_ID": "test_host",
            "SENSOR_ID": "test_sensor",
            "SENSOR_SN": "test_sn",
            "SENSOR_TYPE": "test_type",
            "SENSOR_NAME": "test_name",
            "COT_ACCESS": "test_access",
        }

        events = djicot.functions.handle_frame(frame, config)
        assert len(events) == 1
        for event in events:
            assert djicot.DEFAULT_COT_TYPE in event.decode("utf-8")

    def test_handle_frame_invalid_data(self):
        """Test handle_frame function with invalid data."""
        frame = bytearray(10)
        frame[:2] = b"\x01\x02"  # frame_header
        frame[2] = 0x01  # package_type
        frame[3:5] = struct.pack("H", 10)  # package_length
        frame[5:10] = b"hello"  # invalid data

        config = {
            "DEBUG": True,
            "COT_TYPE": "a-f-G-U-C",
            "COT_STALE": 600,
            "COT_HOST_ID": "test_host",
            "SENSOR_ID": "test_sensor",
            "SENSOR_SN": "test_sn",
            "SENSOR_TYPE": "test_type",
            "SENSOR_NAME": "test_name",
            "COT_ACCESS": "test_access",
        }

        events = djicot.functions.handle_frame(frame, config)
        assert len(events) == 1
        for event in events:
            assert "sensor_id" in event.decode("utf-8")

    def _test_dji_to_cot(self):
        """Test dji_to_cot function with valid data."""
        frame = bytearray(10)
        frame[:2] = b"\x01\x02"  # frame_header
        frame[2] = 0x01  # package_type

        data = bytearray(227)
        data[:64] = b"F4XF7249QM06Q26P".ljust(64, b"\x00")
        data[64:128] = b"DJI Mini 3 pro".ljust(64, b"\x00")
        data[128] = 73
        data[129:137] = struct.pack("d", 37.7749)  # app_lat
        data[137:145] = struct.pack("d", -122.4194)  # app_lon
        data[145:153] = struct.pack("d", 37.7749)  # uas_lat
        data[153:161] = struct.pack("d", -122.4194)  # uas_lon
        data[161:169] = struct.pack("d", 100.0)  # height
        data[169:177] = struct.pack("d", 200.0)  # altitude
        data[177:185] = struct.pack("d", 37.7749)  # home_lat
        data[185:193] = struct.pack("d", -122.4194)  # home_lon
        data[193:201] = struct.pack("d", 2.4)  # freq
        data[201:209] = struct.pack("d", 10.0)  # speed_e
        data[209:217] = struct.pack("d", 5.0)  # speed_n
        data[217:225] = struct.pack("d", 100.0)  # speed_u
        data[225:227] = struct.pack("h", -50)  # rssi

        frame[3:5] = struct.pack("H", len(data))  # package_length
        frame[5 : 5 + len(data)] = data
        print(frame)
        config = {
            "DEBUG": True,
            "COT_TYPE": "a-f-G-U-C",
            "COT_STALE": 600,
            "COT_HOST_ID": "test_host",
            "SENSOR_ID": "test_sensor",
            "SENSOR_SN": "test_sn",
            "SENSOR_TYPE": "test_type",
            "SENSOR_NAME": "test_name",
            "COT_ACCESS": "test_access",
        }

        cot = djicot.functions.dji_uas_to_cot(frame, config)
        print(ET.tostring(cot))
        assert cot is not None
        assert cot.find("detail/__cuas").get("sensor_id") == "test_sensor"
        assert cot.find("detail/__cuas").get("uas_sn") == "serial_number"
        assert cot.find("detail/__cuas").get("uas_type") == "device_type"
        assert cot.find("detail/__cuas").get("freq") == "2.4"
        assert cot.find("detail/__cuas").get("rssi") == "-50"

    def _test_dji_to_cot_invalid_package_type(self):
        """Test dji_to_cot function with invalid package type."""
        data = bytearray(227)
        data[:2] = b"\x01\x00"  # frame_header
        data[2] = 0x02  # invalid package_type
        data[3:5] = struct.pack("H", 227)  # package_length

        config = {
            "DEBUG": True,
            "COT_TYPE": "a-f-G-U-C",
            "COT_STALE": 600,
            "COT_HOST_ID": "test_host",
            "SENSOR_ID": "test_sensor",
            "SENSOR_SN": "test_sn",
            "SENSOR_TYPE": "test_type",
            "SENSOR_NAME": "test_name",
            "COT_ACCESS": "test_access",
        }

        cot = djicot.functions.dji_uas_to_cot(data, config)
        assert cot is None

    def _test_dji_to_cot_missing_lat_lon(self):
        """Test dji_to_cot function with missing lat/lon."""
        data = bytearray(227)
        data[:2] = b"\x01\x00"  # frame_header
        data[2] = 0x01  # package_type
        data[3:5] = struct.pack("H", 227)  # package_length
        data[5:69] = b"serial_number".ljust(64, b"\x00")
        data[69:133] = b"device_type".ljust(64, b"\x00")
        data[133] = 8

        config = {
            "DEBUG": True,
            "COT_TYPE": "a-f-G-U-C",
            "COT_STALE": 600,
            "COT_HOST_ID": "test_host",
            "SENSOR_ID": "test_sensor",
            "SENSOR_SN": "test_sn",
            "SENSOR_TYPE": "test_type",
            "SENSOR_NAME": "test_name",
            "COT_ACCESS": "test_access",
        }

        cot = djicot.functions.dji_to_cot(data, config)
        assert cot is None


def test_parse_frame():
    test_data = b"\x100\x01\xe8\x00F4XF7249QM06Q26P\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00DJI Mini 3 pro\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00I\x01\x81\xbd*I\xe1B@\x9a0j\x97\xda\x9f^\xc0\x01\x81\xbd*I\xe1B@\x9a0j\x97\xda\x9f^\xc0\x00\x00\x00\x00\x00\x00\x00\x00ffffff\xfe?\xce\xa7l:H\xe1B@\xc3q\x82\xdf\xda\x9f^\xc0\x00\x00\x00\x00\x00\xdd\xa2@\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\xd7\xff\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
    dji_type, dji_data = djicot.dji_functions.parse_frame(test_data)
    assert dji_type == 1
    assert len(dji_data) == 227


def test_parse_data():
    test_data = b"\x100\x01\xe8\x00F4XF7249QM06Q26P\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00DJI Mini 3 pro\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00I\x01\x81\xbd*I\xe1B@\x9a0j\x97\xda\x9f^\xc0\x01\x81\xbd*I\xe1B@\x9a0j\x97\xda\x9f^\xc0\x00\x00\x00\x00\x00\x00\x00\x00ffffff\xfe?\xce\xa7l:H\xe1B@\xc3q\x82\xdf\xda\x9f^\xc0\x00\x00\x00\x00\x00\xdd\xa2@\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\xd7\xff\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
    dji_type, dji_data = djicot.dji_functions.parse_frame(test_data)
    assert dji_type == 1
    assert len(dji_data) == 227
    parsed_data = djicot.dji_functions.parse_data(dji_data)
    assert parsed_data.get("serial_number") == "F4XF7249QM06Q26P"
    assert parsed_data.get("device_type") == "DJI Mini 3 pro"
    assert parsed_data.get("device_type_8") == 73
    # assert parsed_data.get("app_lat") == 37.7749
    # assert parsed_data.get("app_lon") == -122.4194
    assert parsed_data.get("uas_lat") == 37.760045378237926
    assert parsed_data.get("uas_lon") == -122.49771676416495
    assert parsed_data.get("height") == 0.0
    assert parsed_data.get("altitude") == 1.9
    assert parsed_data.get("home_lat") == 37.76001673036045
    assert parsed_data.get("home_lon") == -122.49773395289144
    assert parsed_data.get("freq") == 2414.5
    assert parsed_data.get("speed_e") == 0.0
    assert parsed_data.get("speed_n") == 0.0
    assert parsed_data.get("speed_u") == 0.0
    assert parsed_data.get("rssi") == -41


def test_handle_frame():
    test_data = b"\x100\x01\xe8\x00F4XF7249QM06Q26P\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00DJI Mini 3 pro\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00I\x01\x81\xbd*I\xe1B@\x9a0j\x97\xda\x9f^\xc0\x01\x81\xbd*I\xe1B@\x9a0j\x97\xda\x9f^\xc0\x00\x00\x00\x00\x00\x00\x00\x00ffffff\xfe?\xce\xa7l:H\xe1B@\xc3q\x82\xdf\xda\x9f^\xc0\x00\x00\x00\x00\x00\xdd\xa2@\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\xd7\xff\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"

    events = djicot.functions.handle_frame(test_data)
    assert len(events) == 3
    for event in events:
        event = event.decode("utf-8")
        print("-- EVENT --")
        print(event)
        assert "sensor_id" in event
        assert event.startswith("<?xml")
        assert event.endswith("</event>")
        assert "F4XF7249QM06Q26P" in event

        if ".uas" in event:
            assert "DJI.F4XF7249QM06Q26P.uas" in event
            assert "a-u-A-M-H-Q" in event
            assert "DJI Mini 3 pro" in event
            assert "freq=2414.5" in event
            assert 'rssi="-41"' in event

        if ".op" in event:
            assert "DJI.F4XF7249QM06Q26P.op" in event
            assert "a-u-G-U-C" in event
            assert "37.76" in event

        if ".home" in event:
            assert "DJI.F4XF7249QM06Q26P.home" in event
            assert "a-u-G-U-C" in event
            assert "37.76" in event


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright Sensors & Signals LLC https://www.snstac.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""DJICOT Functions."""

import asyncio
import logging
import os
import xml.etree.ElementTree as ET

from configparser import SectionProxy
from typing import Optional, Set, Union

import pytak
import djicot

from .dji_functions import parse_frame, parse_data


APP_NAME = "djicot"
Logger = logging.getLogger(__name__)
Debug = bool(os.getenv("DEBUG", False))


def create_tasks(config: SectionProxy, clitool: pytak.CLITool) -> Set[pytak.Worker,]:
    """Create specific coroutine task set for this application.

    Parameters
    ----------
    config : `SectionProxy`
        Configuration options & values.
    clitool : `pytak.CLITool`
        A PyTAK Worker class instance.

    Returns
    -------
    `set`
        Set of PyTAK Worker classes for this application.
    """
    tasks = set()

    net_queue: asyncio.Queue = asyncio.Queue()

    tasks.add(djicot.DJIWorker(clitool.tx_queue, config, net_queue))
    tasks.add(djicot.NetWorker(net_queue, config))

    return tasks


def dji_uas_to_cot(
    data, config: Union[SectionProxy, dict, None] = None
) -> Optional[ET.Element]:
    """Convert DJI UAS data to CoT."""
    return gen_dji_cot(data, config, "uas")


def dji_op_to_cot(
    data, config: Union[SectionProxy, dict, None] = None
) -> Optional[ET.Element]:
    """Convert DJI Operator data to CoT."""
    return gen_dji_cot(data, config, "op")


def dji_home_to_cot(
    data, config: Union[SectionProxy, dict, None] = None
) -> Optional[ET.Element]:
    """Convert DJI Home data to CoT"""
    return gen_dji_cot(data, config, "home")


def gen_dji_cot(  # NOQA pylint: disable=too-many-locals,too-many-branches,too-many-statements
    data, config: Union[SectionProxy, dict, None] = None, leg="uas"
) -> Optional[ET.Element]:
    """
    Generate a Cursor on Target (CoT) XML Event from DJI data.

    Parameters
    ----------
    data : dict
        Dictionary containing the DJI data.
    config : Union[SectionProxy, dict, None], optional
        Configuration settings, by default None.
    leg : str, optional
        Specifies the leg type, by default "uas".

    Returns
    -------
    Optional[xml.etree.ElementTree.Element]
        The generated CoT XML ElementTree object, or None if latitude or longitude are missing.

    Notes
    -----
    - The function extracts relevant information from the DJI data and config to create a CoT XML element.
    - If latitude or longitude are missing for the "uas" leg, default sensor coordinates are used.
    - The function supports debugging output if "DEBUG" is set in the config.
    - The CoT element includes details such as sensor information, contact callsign, track data, and remarks.
    - The CoT element is generated using the pytak.gen_cot_xml() function.
    """
    config = config or {}
    debug = bool(config.get("DEBUG")) or Debug

    # Extract relevant info for CoT
    lat = data.get(f"{leg}_lat")
    lon = data.get(f"{leg}_lon")
    Logger.debug(f"leg={leg} lat={lat} lon={lon}")

    uas_sn = data.get("serial_number", "")
    uas_type = data.get("device_type", "")

    cot_type: str = str(config.get("COT_TYPE", djicot.DEFAULT_COT_TYPE))
    if leg == "op" or leg == "home":
        cot_type = str("a-u-G-U-C")

    cot_uid = f"DJI.{uas_sn}.{leg}"
    callsign = f"{uas_sn}.{leg}"

    if lat is None or lon is None:
        if leg == "uas":
            lat = config.get("SENSOR_LAT", djicot.DEFAULT_SENSOR_LAT)
            lon = config.get("SENSOR_LON", djicot.DEFAULT_SENSOR_LON)
            cot_type = str("a-u-A-M-H-Q")
            callsign = f"{callsign} (Range)"

        if lat is None or lon is None:
            Logger.debug("No UAS or Sensor lat or lon")
            return None

    remarks_fields: list = []

    cot_stale: int = int(config.get("COT_STALE", pytak.DEFAULT_COT_STALE))
    cot_host_id: str = config.get("COT_HOST_ID", pytak.DEFAULT_HOST_ID)
    sensor_id = str(config.get("SENSOR_ID", djicot.DEFAULT_SENSOR_ID))

    cuas = ET.Element("__cuas")
    cuas.set("sensor_id", sensor_id)
    cuas.set("sensor_sn", str(config.get("SENSOR_SN", djicot.DEFAULT_SENSOR_SN)))
    cuas.set("sensor_type", str(config.get("SENSOR_TYPE", djicot.DEFAULT_SENSOR_TYPE)))
    cuas.set("sensor_name", str(config.get("SENSOR_NAME", djicot.DEFAULT_SENSOR_NAME)))
    cuas.set("cot_host_id", cot_host_id)
    cuas.set("uas_type", uas_type)
    cuas.set("uas_type_8", str(data.get("device_type_8")))
    cuas.set("uas_sn", str(uas_sn))
    cuas.set("freq", str(data.get("freq", 0.0)))
    cuas.set("rssi", str(data.get("rssi", 0)))
    cuas.set("speed_e", str(data.get("speed_e", 0.0)))
    cuas.set("speed_n", str(data.get("speed_n", 0.0)))
    cuas.set("speed_u", str(data.get("speed_u", 0.0)))

    contact: ET.Element = ET.Element("contact")
    contact.set("callsign", callsign)

    track: ET.Element = ET.Element("track")
    track.set("course", data.get("course_point", "9999999.0"))
    track.set("speed", data.get("speed_point", "9999999.0"))

    detail = ET.Element("detail")

    # Remarks should always be the first sub-entity within the Detail entity.
    remarks = ET.Element("remarks")
    remarks_fields.append(f"sn={uas_sn}")
    remarks_fields.append(f"({uas_type})")
    remarks_fields.append(f"freq={data.get('freq', 0.0)}")
    remarks_fields.append(f"rss={data.get('rssi', 0)}")
    remarks_fields.append(f"speed={data.get('speed_e', 0.0)}")
    remarks_fields.append(f"sensor_id={sensor_id}")
    remarks_fields.append(f"host_id={cot_host_id}")
    _remarks = " ".join(list(filter(None, remarks_fields)))
    remarks.text = _remarks
    detail.append(remarks)

    detail.append(contact)
    detail.append(track)
    detail.append(cuas)

    cot_d = {
        "lat": str(lat),
        "lon": str(lon),
        "ce": str(data.get("nac_p", "9999999.0")),
        "le": str(data.get("nac_v", "9999999.0")),
        "hae": str(data.get("alt_geom", "9999999.0")),
        "uid": cot_uid,
        "cot_type": cot_type,
        "stale": cot_stale,
    }
    cot = pytak.gen_cot_xml(**cot_d)
    cot.set("access", config.get("COT_ACCESS", pytak.DEFAULT_COT_ACCESS))

    _detail = cot.findall("detail")[0]
    flowtags = _detail.findall("_flow-tags_")
    detail.extend(flowtags)
    cot.remove(_detail)
    cot.append(detail)

    return cot


def sensor_to_cot(
    data, config: Union[SectionProxy, dict, None] = None
) -> Optional[ET.Element]:
    """Create a CoT Event for the Sensor.

    Parameters
    ----------
    data : dict
        Dictionary containing the sensor data.
    config : Union[SectionProxy, dict, None], optional
        Configuration settings, by default None.

    Returns
    -------
    Optional[xml.etree.ElementTree.Element]
        The generated CoT XML ElementTree object, or None if latitude or longitude are missing.

    Notes
    -----
    - The function extracts relevant information from the sensor data and config to create a CoT XML element.
    - If latitude or longitude are missing, the function returns None.
    - The CoT element includes details such as sensor information, contact callsign, and remarks.
    - The CoT element is generated using the pytak.gen_cot_xml() function.
    """
    config = config or {}
    debug = bool(config.get("DEBUG"))

    lat = config.get("SENSOR_LAT", djicot.DEFAULT_SENSOR_LAT)
    lon = config.get("SENSOR_LON", djicot.DEFAULT_SENSOR_LON)

    if lat is None or lon is None:
        Logger.debug("No Sensor Lat/Lon")
        return None

    sensor_id = config.get("SENSOR_ID", djicot.DEFAULT_SENSOR_ID)
    sensor_sn = config.get("SENSOR_SN", djicot.DEFAULT_SENSOR_SN)
    sensor_type = config.get("SENSOR_TYPE", djicot.DEFAULT_SENSOR_TYPE)
    cot_host_id: str = config.get("COT_HOST_ID", pytak.DEFAULT_HOST_ID)

    cot_uid = config.get("SENSOR_UID", f"CUAS-{sensor_type}-{sensor_sn}-{cot_host_id}")
    callsign = config.get("SENSOR_CALLSIGN", f"CUAS-{sensor_type}-{sensor_sn}")

    cot_type = config.get("SENSOR_COT_TYPE", djicot.DEFAULT_SENSOR_COT_TYPE)
    cot_stale = config.get("SENSOR_STALE", djicot.DEFAULT_SENSOR_STALE)

    cuas = ET.Element("__cuas")
    cuas.set("sensor_id", sensor_id)
    cuas.set("sensor_sn", sensor_sn)
    cuas.set("sensor_type", sensor_type)
    cuas.set("cot_host_id", cot_host_id)

    contact: ET.Element = ET.Element("contact")
    contact.set("callsign", callsign)

    detail = ET.Element("detail")

    # Remarks should always be the first sub-entity within the Detail entity.
    remarks = ET.Element("remarks")
    remarks.text = f"sensor_id={sensor_id} sensor_sn={sensor_sn} sensor_type={sensor_type} cot_host_id={cot_host_id}: {data}"

    detail.append(remarks)
    detail.append(contact)
    detail.append(cuas)

    cot_d = {
        "lat": str(lat),
        "lon": str(lon),
        "ce": str(config.get("SENSOR_CE", "9999999.0")),
        "le": str(config.get("SENSOR_LE", "9999999.0")),
        "hae": str(config.get("SENSOR_HAE", "9999999.0")),
        "uid": cot_uid,
        "cot_type": cot_type,
        "stale": cot_stale,
    }
    cot = pytak.gen_cot_xml(**cot_d)
    cot.set("access", config.get("COT_ACCESS", pytak.DEFAULT_COT_ACCESS))

    _detail = cot.findall("detail")[0]
    flowtags = _detail.findall("_flow-tags_")
    detail.extend(flowtags)
    cot.remove(_detail)
    cot.append(detail)

    return cot


def xml_to_cot(
    data: dict, config: Union[SectionProxy, dict, None] = None, func=None
) -> Optional[bytes]:
    """Convert data to a CoT XML string using the specified function."""
    cot: Optional[ET.Element] = getattr(djicot.functions, func)(data, config)
    return (
        b"\n".join([pytak.DEFAULT_XML_DECLARATION, ET.tostring(cot)]) if cot else None
    )


def handle_frame(
    frame: bytearray, config: Union[SectionProxy, dict, None] = None
) -> list:
    """
    Handles a DJI frame by parsing it and converting the parsed data to CoT (Cursor on Target) events.

    Args:
        frame (bytearray): The DJI frame to be handled.
        config (Union[SectionProxy, dict, None], optional): Configuration settings. Defaults to None.

    Returns:
        list: A list of CoT events generated from the parsed DJI data.

    Raises:
        Exception: If there is an error parsing the DJI frame or data.
    """
    config = config or {}
    events = []
    package_type = None
    data = None
    parsed_data = {}
    debug = bool(config.get("DEBUG")) or Debug

    try:
        package_type, data = parse_frame(frame)
    except Exception as exc:
        Logger.debug(f"Error parsing DJI frame: {exc}")

    if package_type != 0x01:
        Logger.debug(f"Invalid DJI package type: {package_type}")

    if not data:
        Logger.debug("No DJI data")
    else:
        try:
            parsed_data = parse_data(data)
        except Exception as exc:
            Logger.debug(f"Error parsing DJI data: {exc}")

    Logger.debug(f"Parsed DJI data: {parsed_data}")

    funcs = ["dji_uas_to_cot", "dji_op_to_cot", "dji_home_to_cot"]
    for func in funcs:
        event: Optional[bytes] = xml_to_cot(parsed_data, config, func)
        if event:
            events.append(event)

    return events



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright Sensors & Signals LLC https://www.snstac.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""DJI OcuSync Decoding Functions."""

import logging
import os
import struct

Logger = logging.getLogger(__name__)

DJI_PAYLOAD = {
    "serial_number": None,
    "device_type": None,
    "device_type_8": None,
    "op_lat": None,
    "op_lon": None,
    "uas_lat": None,
    "uas_lon": None,
    "height": None,
    "altitude": None,
    "home_lat": None,
    "home_lon": None,
    "freq": None,
    "speed_e": None,
    "speed_n": None,
    "speed_u": None,
    "rssi": None,
    "software": "DJICOT",
}


def parse_frame(frame):
    """
    Parses a given OcuSync frame and extracts its components.

    Args:
        frame (bytes): The input frame to be parsed.

    Returns:
        tuple: A tuple containing the package type (int) and the data (bytes).

    The frame is expected to have the following structure:
        - The first 2 bytes are the frame header.
        - The 3rd byte is the package type.
        - The 4th and 5th bytes represent the length of the package.
        - The remaining bytes contain the data.

    The function logs the frame, frame header, package type, length bytes, package length, and data for debugging purposes.
    """
    Logger.debug("frame=", frame)

    frame_header = frame[:2]
    package_type = frame[2]
    length_bytes = frame[3:5]
    package_length = struct.unpack("H", length_bytes)[0]
    data = frame[5 : 5 + package_length - 5]

    Logger.debug("frame_header=", frame_header)
    Logger.debug("package_type=", package_type)
    Logger.debug("length_bytes=", length_bytes)
    Logger.debug("package_length=", package_length)
    Logger.debug("data=", data)

    return package_type, data


def parse_data(data):
    payload = DJI_PAYLOAD.copy()
    try:
        payload = {
            "serial_number": data[:64].decode("utf-8").rstrip("\x00"),
            "device_type": data[64:128].decode("utf-8").rstrip("\x00"),
            "device_type_8": data[128],
            "op_lat": struct.unpack("d", data[129:137])[0],
            "op_lon": struct.unpack("d", data[137:145])[0],
            "uas_lat": struct.unpack("d", data[145:153])[0],
            "uas_lon": struct.unpack("d", data[153:161])[0],
            "height": struct.unpack("d", data[161:169])[0],
            "altitude": struct.unpack("d", data[169:177])[0],
            "home_lat": struct.unpack("d", data[177:185])[0],
            "home_lon": struct.unpack("d", data[185:193])[0],
            "freq": struct.unpack("d", data[193:201])[0],
            "speed_e": struct.unpack("d", data[201:209])[0],
            "speed_n": struct.unpack("d", data[209:217])[0],
            "speed_u": struct.unpack("d", data[217:225])[0],
            "rssi": struct.unpack("h", data[225:227])[0],
        }
    except UnicodeDecodeError as exc:
        if bool(os.getenv("DEBUG")):
            print(f"UnicodeDecodeError: {exc}")
        # If we fail to decode, it may indicate encrypted or partial data
        payload = {
            "device_type": "Unknown DJI OcuSync Format (Encrypted or Partial Data)",
            "device_type_8": 255,
        }

    return payload



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright Sensors & Signals LLC https://www.snstac.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""DJICOT Class Definitions."""

import asyncio

import time

from configparser import SectionProxy
from typing import Optional, Union
from urllib.parse import urlparse, ParseResult

from pytak import QueueWorker

from djicot import xml_to_cot, handle_frame, DEFAULT_FEED_URL, DEFAULT_READ_BYTES


class DJIWorker(QueueWorker):
    """
    DJIWorker is responsible for reading DJI data from inputs, serializing it to
    CoT (Cursor on Target) format, and placing it on the transmission (TX) queue.

    Attributes:
        tx_queue (asyncio.Queue): The queue to which serialized CoT data is sent.
        config (Union[SectionProxy, dict]): Configuration settings for the worker.
        net_queue (asyncio.Queue): The queue from which raw DJI data is received.
    """

    def __init__(
        self,
        tx_queue: asyncio.Queue,
        config: Union[SectionProxy, dict],
        net_queue: asyncio.Queue,
    ) -> None:
        """Initializes the DJIWorker with the given TX queue, configuration, and network queue."""
        super().__init__(tx_queue, config)
        self.net_queue = net_queue

    async def handle_data(self, data) -> None:
        """Processes raw DJI data, converts it to CoT format, and places it on the TX queue."""
        events = handle_frame(data, self.config)
        for event in events:
            await self.put_queue(event)

    async def hello_event(self, init=False):
        """Sends a "hello world" style event to the TX queue. This event is sent periodically or on initialization."""
        timer = int(time.time()) % 60 == 0
        if init or timer:
            data = f"init={init} timer={timer}"
            event: Optional[bytes] = xml_to_cot(data, self.config, "sensor_to_cot")
            await self.put_queue(event)

    async def run(self, _=-1) -> None:
        """Main execution loop of the worker. Sends periodic hello events and processes incoming data from the network queue."""
        self._logger.info("Running %s", self.__class__)

        if not self.config.get("PYTAK_NO_HELLO", False):
            await self.hello_event(init=True)

        while True:
            await self.hello_event()
            received = await self.net_queue.get()
            if not received:
                continue
            await self.handle_data(received)


class NetWorker(QueueWorker):  # pylint: disable=too-few-public-methods
    """
    A worker class that reads data from a network connection and puts it on a queue.
    """

    async def handle_data(self, data) -> None:
        """Asynchronously handles incoming data by placing it on the queue."""
        self.queue.put_nowait(data)

    async def run(self, _=-1) -> None:
        """Asynchronously runs the main loop to read data from the network and add it to the queue."""
        url: ParseResult = urlparse(self.config.get("FEED_URL", DEFAULT_FEED_URL))

        self._logger.info("Running %s for %s", self.__class__, url.geturl())

        host, port = url.netloc.split(":")

        self._logger.debug("host=%s port=%s", host, port)

        reader, _ = await asyncio.open_connection(host, port)

        read_bytes = self.config.get("READ_BYTES", DEFAULT_READ_BYTES)
        self._logger.debug("read_bytes=%s", read_bytes)

        while True:
            received = await reader.read(read_bytes)
            await self.handle_data(received)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright Sensors & Signals LLC https://www.snstac.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""DJICOT Constants."""

# Feed URL to use out of the box, in this case the JSON from the local filesystem.
DEFAULT_FEED_URL: str = "tcp://192.168.1.10:41030"

DEFAULT_COT_TYPE: str = "a-u-A-M-H-Q"

DEFAULT_SENSOR_LAT: str = "0.0"
DEFAULT_SENSOR_LON: str = "0.0"
DEFAULT_SENSOR_ID: str = "DJICOT"
DEFAULT_SENSOR_NAME: str = "DJICOT"
DEFAULT_SENSOR_DETAIL: str = "DJICOT is a DJI Drone ID to TAK Gateway"
DEFAULT_SENSOR_STALE: str = "600"
DEFAULT_SENSOR_TYPE: str = "Drone ID"
DEFAULT_SENSOR_UID: str = "DJICOT-0001"
DEFAULT_SENSOR_CONTACT: str = "info@snstac.com"
DEFAULT_SENSOR_COT_TYPE: str = "a-f-G-E-S-E"
DEFAULT_SENSOR_SN: str = "0001"
DEFAULT_SENSOR_HAE: str = "0"
DEFAULT_SENSOR_CE: str = "0"
DEFAULT_SENSOR_LE: str = "0"
DEFAULT_SENSOR_ICON: str = "u-av-a-f-o"
DEFAULT_SENSOR_COLOR: str = "#FF0000"
DEFAULT_SENSOR_ICONSET: str = "2525c"

DEFAULT_READ_BYTES: int = 1024
DEFAULT_READ_TIMEOUT: int = 1



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright Sensors & Signals LLC https://www.snstac.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""
DJI to TAK Gateway.

This module serves as the entry point for the DJI to TAK Gateway application.
It imports and exposes constants, functions, and classes necessary for the
operation of the gateway.
"""

__version__ = "1.0.0"

from .constants import (  # NOQA
    DEFAULT_FEED_URL,
    DEFAULT_COT_TYPE,
    DEFAULT_SENSOR_LAT,
    DEFAULT_SENSOR_LON,
    DEFAULT_SENSOR_ID,
    DEFAULT_SENSOR_NAME,
    DEFAULT_SENSOR_DETAIL,
    DEFAULT_SENSOR_STALE,
    DEFAULT_SENSOR_TYPE,
    DEFAULT_SENSOR_UID,
    DEFAULT_SENSOR_CONTACT,
    DEFAULT_SENSOR_COT_TYPE,
    DEFAULT_SENSOR_SN,
    DEFAULT_READ_BYTES,
)

from .functions import create_tasks, xml_to_cot, handle_frame  # NOQA

from .classes import DJIWorker, NetWorker  # NOQA
